@@ -551,6 +551,23 @@ finalize_it:
 
 static void
 RunCancelCleanup(void *arg)
+{
+	nspoll_t **ppPoll = (nspoll_t**) arg;
+
+	if (*ppPoll != NULL)
+		nspoll.Destruct(ppPoll);
+
+	/* Wait for any running workers to finish */
+	pthread_mutex_lock(&wrkrMut);
+	DBGPRINTF("tcpsrv terminating, waiting for %d workers\n", wrkrRunning);
+	while(wrkrRunning > 0) {
+		pthread_cond_wait(&wrkrIdle, &wrkrMut);
+	}
+	pthread_mutex_unlock(&wrkrMut);
+}
+
+static void
+RunSelectCancelCleanup(void *arg)
 {
 	nssel_t **ppSel = (nssel_t**) arg;
 
@@ -684,7 +701,6 @@ wrkr(void *const myself)
 			// we need to query me->opSrv to avoid clang static
 			// analyzer false positive! -- rgerhards, 2017-10-23
 			assert(glbl.GetGlobalInputTermState() == 1);
-			--wrkrRunning;
 			break;
 		}
 		pthread_mutex_unlock(&wrkrMut);
@@ -703,6 +719,20 @@ wrkr(void *const myself)
 	return NULL;
 }
 
+/* This has been factored out from processWorkset() because
+ * pthread_cleanup_push() invokes setjmp() and this triggers the -Wclobbered
+ * warning for the iRet variable.
+ */
+static void
+waitForWorkers(void)
+{
+	pthread_mutex_lock(&wrkrMut);
+	pthread_cleanup_push(mutexCancelCleanup, &wrkrMut);
+	while(wrkrRunning > 0) {
+		pthread_cond_wait(&wrkrIdle, &wrkrMut);
+	}
+	pthread_cleanup_pop(1);
+}
 
 /* Process a workset, that is handle io. We become activated
  * from either select or epoll handler. We split the workload
@@ -725,6 +755,11 @@ processWorkset(tcpsrv_t *pThis, nspoll_t *pPoll, int numEntries, nsd_epworkset_t
 			/* process self, save context switch */
 			iRet = processWorksetItem(pThis, pPoll, workset[numEntries-1].id, workset[numEntries-1].pUsr);
 		} else {
+			/* No cancel handler needed here, since no cancellation
+			 * points are executed while wrkrMut is locked.
+			 *
+			 * Re-evaluate this if you add a DBGPRINTF or something!
+			 */
 			pthread_mutex_lock(&wrkrMut);
 			/* check if there is a free worker */
 			for(i = 0 ; (i < wrkrMax) && ((wrkrInfo[i].pSrv != NULL) || (wrkrInfo[i].enabled == 0)) ; ++i)
@@ -758,11 +793,7 @@ processWorkset(tcpsrv_t *pThis, nspoll_t *pPoll, int numEntries, nsd_epworkset_t
 		 * rest of this module can not handle the concurrency introduced
 		 * by workers running during the epoll call.
 		 */
-		pthread_mutex_lock(&wrkrMut);
-		while(wrkrRunning > 0) {
-			pthread_cond_wait(&wrkrIdle, &wrkrMut);
-		}
-		pthread_mutex_unlock(&wrkrMut);
+		waitForWorkers();
 	}
 
 finalize_it:
@@ -790,11 +821,7 @@ RunSelect(tcpsrv_t *pThis, nsd_epworkset_t workset[], size_t sizeWorkset)
 
 	ISOBJ_TYPE_assert(pThis, tcpsrv);
 
-	/* this is an endless loop - it is terminated by the framework canelling
-	 * this thread. Thus, we also need to instantiate a cancel cleanup handler
-	 * to prevent us from leaking anything. -- rgerhards, 20080-04-24
-	 */
-	pthread_cleanup_push(RunCancelCleanup, (void*) &pSel);
+	pthread_cleanup_push(RunSelectCancelCleanup, (void*) &pSel);
 	while(1) {
 		CHKiRet(nssel.Construct(&pSel));
 		if(pThis->pszDrvrName != NULL)
@@ -874,8 +901,7 @@ finalize_it: /* this is a very special case - this time only we do not exit the
 		}
 	}
 
-	/* note that this point is usually not reached */
-	pthread_cleanup_pop(1); /* remove cleanup handler */
+	pthread_cleanup_pop(1); /* execute and remove cleanup handler */
 
 	RETiRet;
 }
@@ -892,7 +918,7 @@ Run(tcpsrv_t *pThis)
 {
 	DEFiRet;
 	int i;
-	int bFailed = FALSE; /* If set to TRUE, accept failed already */
+	int bFailed; /* If set to TRUE, accept failed */
 	nsd_epworkset_t workset[128]; /* 128 is currently fixed num of concurrent requests */
 	int numEntries;
 	nspoll_t *pPoll = NULL;
@@ -910,10 +936,13 @@ Run(tcpsrv_t *pThis)
 	}
 	d_pthread_mutex_unlock(&wrkrMut);
 
-	/* this is an endless loop - it is terminated by the framework canelling
-	 * this thread. Thus, we also need to instantiate a cancel cleanup handler
-	 * to prevent us from leaking anything. -- rgerhards, 20080-04-24
+	/* We try to terminate cleanly, but install a cancellation clean-up
+	 * handler in case we are cancelled.
 	 */
+	pthread_cleanup_push(RunCancelCleanup, (void*) &pPoll);
+	/* Reset iRet to avoid warning about it being clobbered by longjmp */
+	iRet = RS_RET_OK;
+
 	if((localRet = nspoll.Construct(&pPoll)) == RS_RET_OK) {
 		if(pThis->pszDrvrName != NULL)
 			CHKiRet(nspoll.SetDrvrName(pPoll, pThis->pszDrvrName));
@@ -938,6 +967,7 @@ Run(tcpsrv_t *pThis)
 		DBGPRINTF("Added listener %d\n", i);
 	}
 
+	bFailed = FALSE;
 	while(glbl.GetGlobalInputTermState() == 0) {
 		numEntries = sizeof(workset)/sizeof(nsd_epworkset_t);
 		localRet = nspoll.Wait(pPoll, -1, &numEntries, workset);
@@ -978,8 +1008,8 @@ Run(tcpsrv_t *pThis)
 	}
 
 finalize_it:
-	if(pPoll != NULL)
-		nspoll.Destruct(&pPoll);
+	pthread_cleanup_pop(1);
+
 	RETiRet;
 }
 
